import { gql, useLazyQuery, useMutation, useQuery } from "@apollo/client";
import { useCallback, useState } from "react";
import { CreateDraftArticleInput, DraftArticle } from "../../generated/graphql";

export interface useDraftsProps {
  key: string;
}

export interface useDraftsReturn {
  drafts?: DraftArticle[];
  draft?: DraftArticle;
  listLoading: boolean;
  itemLoading: boolean;
  getDraft: (id: string) => Promise<DraftArticle>;
  putDraft: (input: CreateDraftArticleInput) => Promise<DraftArticle>;
}

export const useDrafts = (key: string): useDraftsReturn => {
  const { data: { drafts, lastDraft: last } = {}, loading: listLoading } =
    useQuery<
      { drafts: DraftArticle[]; lastDraft: DraftArticle },
      { key: string }
    >(
      gql`
        query DraftArticles($key: String!) {
          drafts: listDraftForArticle(key: $key) {
            id
            key
            automatically
            updatedAt
          }
          lastDraft: lastDraftForArticle(key: $key) {
            id
            key
            payload
            automatically
            updatedAt
          }
        }
      `,
      {
        variables: { key },
      }
    );
  const [
    get,
    { data: { draft } = { draft: undefined }, loading: itemLoading },
  ] = useLazyQuery<
    {
      draft: DraftArticle;
    },
    {
      id: string;
    }
  >(
    gql`
      query DraftArticles($id: String!) {
        draft: lastDraftForArticle(id: $id) {
          id
          key
          payload
          automatically
          updatedAt
        }
      }
    `
  );

  const [put] = useMutation<
    {
      draft: DraftArticle;
    },
    {
      draft: CreateDraftArticleInput;
    }
  >(
    gql`
      mutation PutDraftForArticle($draft: CreateDraftArticleInput!) {
        draft: putDraftForArticle(draft: $draft) {
          id
          key
          payload
          automatically
          updatedAt
        }
      }
    `,
    {
      update(cache, { data }) {
        if (data?.draft) {
          const newDraftRef = cache.writeFragment({
            data: data.draft,
            fragment: gql`
              fragment NewDraftArticle on DraftArticle {
                id
                key
                payload
                automatically
                updatedAt
              }
            `,
          });
          cache.modify({
            fields: {
              drafts(existingDrafts: DraftArticle[] = [], { readField }) {
                return [
                  newDraftRef,
                  ...existingDrafts.filter(
                    (draft: any) => readField("id", draft) !== data.draft.id
                  ),
                ];
              },
              lastDraft() {
                return newDraftRef;
              },
            },
          });
        }
      },
    }
  );

  const getDraft = useCallback(
    async (id: string) => {
      return get({ variables: { id } }).then(({ data }) => data!.draft);
    },
    [get]
  );

  const putDraft = useCallback(
    async (input: CreateDraftArticleInput) => {
      return put({ variables: { draft: input } }).then(
        ({ data }) => data!.draft
      );
    },
    [put]
  );

  return {
    drafts,
    draft: draft ?? last,
    listLoading,
    itemLoading,
    getDraft,
    putDraft,
  };
};
